@@ -22,7 +22,7 @@ module Mail
     end
 
     def to_s
-      decoded
+      decoded.to_s
     end
 
     def default
@@ -34,7 +34,7 @@ module Mail
     end
 
     def responsible_for?( val )
-      name.to_s.downcase == val.to_s.downcase
+      name.to_s.casecmp(val.to_s) == 0
     end
 
     private
@@ -47,5 +47,11 @@ module Mail
       end
     end
 
+    FILENAME_RE = /\b(filename|name)=([^;"\r\n]+\s[^;"\r\n]+)/
+    def ensure_filename_quoted(value)
+      if value.is_a?(String)
+        value.sub! FILENAME_RE, '\1="\2"'
+      end
+    end
   end
 end
