@@ -108,6 +108,9 @@ module Mail
       @charset = 'UTF-8'
       @defaulted_charset = true
 
+      @smtp_envelope_from = nil
+      @smtp_envelope_to = nil
+
       @perform_deliveries = true
       @raise_delivery_errors = true
 
@@ -122,7 +125,7 @@ module Mail
       if args.flatten.first.respond_to?(:each_pair)
         init_with_hash(args.flatten.first)
       else
-        init_with_string(args.flatten[0].to_s.strip)
+        init_with_string(args.flatten[0].to_s)
       end
 
       if block_given?
@@ -169,7 +172,7 @@ module Mail
     #   obj.mail.deliver
     #
     # Would cause Mail to call obj.deliver_mail passing itself as a parameter,
-    # which then can just yield and let Mail do it's own private do_delivery
+    # which then can just yield and let Mail do its own private do_delivery
     # method.
     attr_accessor :delivery_handler
 
@@ -938,8 +941,8 @@ module Mail
     #
     # Example:
     #
-    #  mail.sender = 'Mikel <mikel@test.lindsaar.net>'
-    #  mail.sender #=> 'mikel@test.lindsaar.net'
+    #  mail.resent_sender = 'Mikel <mikel@test.lindsaar.net>'
+    #  mail.resent_sender #=> 'mikel@test.lindsaar.net'
     def resent_sender=( val )
       header[:resent_sender] = val
     end
@@ -1023,6 +1026,82 @@ module Mail
       header[:sender] = val
     end
 
+    # Returns the SMTP Envelope From value of the mail object, as a single
+    # string of an address spec.
+    #
+    # Defaults to Return-Path, Sender, or the first From address.
+    #
+    # Example:
+    #
+    #  mail.smtp_envelope_from = 'Mikel <mikel@test.lindsaar.net>'
+    #  mail.smtp_envelope_from #=> 'mikel@test.lindsaar.net'
+    #
+    # Also allows you to set the value by passing a value as a parameter
+    #
+    # Example:
+    #
+    #  mail.smtp_envelope_from 'Mikel <mikel@test.lindsaar.net>'
+    #  mail.smtp_envelope_from #=> 'mikel@test.lindsaar.net'
+    def smtp_envelope_from( val = nil )
+      if val
+        self.smtp_envelope_from = val
+      else
+        @smtp_envelope_from || return_path || sender || from_addrs.first
+      end
+    end
+
+    # Sets the From address on the SMTP Envelope.
+    #
+    # Example:
+    #
+    #  mail.smtp_envelope_from = 'Mikel <mikel@test.lindsaar.net>'
+    #  mail.smtp_envelope_from #=> 'mikel@test.lindsaar.net'
+    def smtp_envelope_from=( val )
+      @smtp_envelope_from = val
+    end
+
+    # Returns the SMTP Envelope To value of the mail object.
+    #
+    # Defaults to #destinations: To, Cc, and Bcc addresses.
+    #
+    # Example:
+    #
+    #  mail.smtp_envelope_to = 'Mikel <mikel@test.lindsaar.net>'
+    #  mail.smtp_envelope_to #=> 'mikel@test.lindsaar.net'
+    #
+    # Also allows you to set the value by passing a value as a parameter
+    #
+    # Example:
+    #
+    #  mail.smtp_envelope_to ['Mikel <mikel@test.lindsaar.net>', 'Lindsaar <lindsaar@test.lindsaar.net>']
+    #  mail.smtp_envelope_to #=> ['mikel@test.lindsaar.net', 'lindsaar@test.lindsaar.net']
+    def smtp_envelope_to( val = nil )
+      if val
+        self.smtp_envelope_to = val
+      else
+        @smtp_envelope_to || destinations
+      end
+    end
+
+    # Sets the To addresses on the SMTP Envelope.
+    #
+    # Example:
+    #
+    #  mail.smtp_envelope_to = 'Mikel <mikel@test.lindsaar.net>'
+    #  mail.smtp_envelope_to #=> 'mikel@test.lindsaar.net'
+    #
+    #  mail.smtp_envelope_to = ['Mikel <mikel@test.lindsaar.net>', 'Lindsaar <lindsaar@test.lindsaar.net>']
+    #  mail.smtp_envelope_to #=> ['mikel@test.lindsaar.net', 'lindsaar@test.lindsaar.net']
+    def smtp_envelope_to=( val )
+      @smtp_envelope_to =
+        case val
+        when Array, NilClass
+          val
+        else
+          [val]
+        end
+    end
+
     # Returns the decoded value of the subject field, as a single string.
     #
     # Example:
@@ -1411,7 +1490,7 @@ module Mail
 
     # Returns the MIME media type of part we are on, this is taken from the content-type header
     def mime_type
-      content_type ? header[:content_type].string : nil rescue nil
+      has_content_type? ? header[:content_type].string : nil rescue nil
     end
 
     def message_content_type
@@ -1422,7 +1501,7 @@ module Mail
     # Returns the character set defined in the content type field
     def charset
       if @header
-        content_type ? content_type_parameters['charset'] : @charset
+        has_content_type? ? content_type_parameters['charset'] : @charset
       else
         @charset
       end
@@ -1516,7 +1595,7 @@ module Mail
 
     # Returns an AttachmentsList object, which holds all of the attachments in
     # the receiver object (either the entier email or a part within) and all
-    # of it's descendants.
+    # of its descendants.
     #
     # It also allows you to add attachments to the mail object directly, like so:
     #
@@ -1559,9 +1638,7 @@ module Mail
     # Accessor for html_part
     def html_part(&block)
       if block_given?
-        @html_part = Mail::Part.new(&block)
-        add_multipart_alternate_header unless html_part.blank?
-        add_part(@html_part)
+        self.html_part = Mail::Part.new(:content_type => 'text/html', &block)
       else
         @html_part || find_first_mime_type('text/html')
       end
@@ -1570,9 +1647,7 @@ module Mail
     # Accessor for text_part
     def text_part(&block)
       if block_given?
-        @text_part = Mail::Part.new(&block)
-        add_multipart_alternate_header unless html_part.blank?
-        add_part(@text_part)
+        self.text_part = Mail::Part.new(:content_type => 'text/plain', &block)
       else
         @text_part || find_first_mime_type('text/plain')
       end
@@ -1581,36 +1656,54 @@ module Mail
     # Helper to add a html part to a multipart/alternative email.  If this and
     # text_part are both defined in a message, then it will be a multipart/alternative
     # message and set itself that way.
-    def html_part=(msg = nil)
+    def html_part=(msg)
+      # Assign the html part and set multipart/alternative if there's a text part.
       if msg
         @html_part = msg
-      else
-        @html_part = Mail::Part.new('Content-Type: text/html;')
+        @html_part.content_type = 'text/html' unless @html_part.has_content_type?
+        add_multipart_alternate_header if text_part
+        add_part @html_part
+
+      # If nil, delete the html part and back out of multipart/alternative.
+      elsif @html_part
+        parts.delete_if { |p| p.object_id == @html_part.object_id }
+        @html_part = nil
+        if text_part
+          self.content_type = nil
+          body.boundary = nil
+        end
       end
-      add_multipart_alternate_header unless text_part.blank?
-      add_part(@html_part)
     end
 
     # Helper to add a text part to a multipart/alternative email.  If this and
     # html_part are both defined in a message, then it will be a multipart/alternative
     # message and set itself that way.
-    def text_part=(msg = nil)
+    def text_part=(msg)
+      # Assign the text part and set multipart/alternative if there's an html part.
       if msg
         @text_part = msg
-      else
-        @text_part = Mail::Part.new('Content-Type: text/plain;')
+        @text_part.content_type = 'text/plain' unless @text_part.has_content_type?
+        add_multipart_alternate_header if html_part
+        add_part @text_part
+
+      # If nil, delete the text part and back out of multipart/alternative.
+      elsif @text_part
+        parts.delete_if { |p| p.object_id == @text_part.object_id }
+        @text_part = nil
+        if html_part
+          self.content_type = nil
+          body.boundary = nil
+        end
       end
-      add_multipart_alternate_header unless html_part.blank?
-      add_part(@text_part)
     end
 
     # Adds a part to the parts list or creates the part list
     def add_part(part)
       if !body.multipart? && !self.body.decoded.blank?
-         @text_part = Mail::Part.new('Content-Type: text/plain;')
-         @text_part.body = body.decoded
-         self.body << @text_part
-         add_multipart_alternate_header
+        @text_part = Mail::Part.new('Content-Type: text/plain;')
+        @text_part.body = body.decoded
+        self.body << @text_part
+        add_multipart_alternate_header
       end
       add_boundary
       self.body << part
@@ -1646,7 +1739,7 @@ module Mail
     #  m.add_file(:filename => 'filename.png', :content => File.read('/path/to/file.jpg'))
     #
     # Note also that if you add a file to an existing message, Mail will convert that message
-    # to a MIME multipart email, moving whatever plain text body you had into it's own text
+    # to a MIME multipart email, moving whatever plain text body you had into its own text
     # plain part.
     #
     # Example:
@@ -1683,7 +1776,7 @@ module Mail
       self.body << text_part
     end
 
-    # Encodes the message, calls encode on all it's parts, gets an email message
+    # Encodes the message, calls encode on all its parts, gets an email message
     # ready to send
     def ready_to_send!
       identify_and_set_transfer_encoding
@@ -1875,15 +1968,13 @@ module Mail
     # Additionally, I allow for the case where someone might have put whitespace
     # on the "gap line"
     def parse_message
-      header_part, body_part = raw_source.split(/#{CRLF}#{WSP}*#{CRLF}/m, 2)
-#      index = raw_source.index(/#{CRLF}#{WSP}*#{CRLF}/m, 2)
-#      self.header = (index) ? header_part[0,index] : nil
-#      lazy_body ( [raw_source, index+1])
+      header_part, body_part = raw_source.lstrip.split(/#{CRLF}#{CRLF}|#{CRLF}#{WSP}*#{CRLF}(?!#{WSP})/m, 2)
       self.header = header_part
       self.body   = body_part
     end
 
     def raw_source=(value)
+      value.force_encoding("binary") if RUBY_VERSION >= "1.9.1"
       @raw_source = value.to_crlf
     end
 
@@ -1906,17 +1997,18 @@ module Mail
 
 
     def process_body_raw
-       @body = Mail::Body.new(@body_raw)
-       @body_raw = nil
-       separate_parts if @separate_parts
+      @body = Mail::Body.new(@body_raw)
+      @body_raw = nil
+      separate_parts if @separate_parts
 
-       add_encoding_to_body
+      add_encoding_to_body
     end
 
     def set_envelope_header
-      if match_data = raw_source.to_s.match(/\AFrom\s(#{TEXT}+)#{CRLF}(.*)/m)
+      raw_string = raw_source.to_s
+      if match_data = raw_source.to_s.match(/\AFrom\s(#{TEXT}+)#{CRLF}/m)
         set_envelope(match_data[1])
-        self.raw_source = match_data[2]
+        self.raw_source = raw_string.sub(match_data[0], "") 
       end
     end
 
@@ -1939,16 +2031,19 @@ module Mail
     end
 
     def add_required_fields
-      add_multipart_mixed_header    unless !body.multipart?
-      body = nil                    if body.nil?
-      add_message_id                unless (has_message_id? || self.class == Mail::Part)
-      add_date                      unless has_date?
-      add_mime_version              unless has_mime_version?
+      add_required_message_fields
+      add_multipart_mixed_header    if body.multipart?
       add_content_type              unless has_content_type?
       add_charset                   unless has_charset?
       add_content_transfer_encoding unless has_content_transfer_encoding?
     end
 
+    def add_required_message_fields
+      add_date          unless has_date?
+      add_mime_version  unless has_mime_version?
+      add_message_id    unless has_message_id?
+    end
+
     def add_multipart_alternate_header
       header['content-type'] = ContentTypeField.with_boundary('multipart/alternative').value
       header['content_type'].parameters[:charset] = @charset
@@ -2047,7 +2142,7 @@ module Mail
         else
           if encoding = Encoding.find(charset) rescue nil
             body_text.force_encoding(encoding)
-            return body_text.encode(Encoding::UTF_8)
+            return body_text.encode(Encoding::UTF_8, :undef => :replace, :invalid => :replace, :replace => '')
           end
         end
       end
